import type { Alignment } from './types/api';
/**
 * Pads a string to the left and/or right to position the subject
 * text in a desired alignment within a container.
 */
export declare const alignString: (subject: string, containerWidth: number, alignment: Alignment) => string;
