import type { SpanningCellConfig } from './types/api';
import type { ColumnConfig, RangeConfig } from './types/internal';
export declare const makeRangeConfig: (spanningCellConfig: SpanningCellConfig, columnsConfig: ColumnConfig[]) => RangeConfig;
