const types = require('./dist/types')
const util = require('./dist/util')

module.exports = types.Schema
module.exports.nullOptions = types.nullOptions
module.exports.strOptions = types.strOptions
module.exports.stringify = util.stringifyString

require('./dist/legacy-exports').warnFileDeprecation(__filename)
