import type { BaseConfig, Row } from './types/internal';
export declare const padString: (input: string, paddingLeft: number, paddingRight: number) => string;
export declare const padTableData: (rows: Row[], config: BaseConfig) => Row[];
