/**
 * Wrap a single cell value into a list of lines
 *
 * Always wraps on newlines, for the remainder uses either word or string wrapping
 * depending on user configuration.
 *
 */
export declare const wrapCell: (cellValue: string, cellWidth: number, useWrapWord: boolean) => string[];
