"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.calculateOutputColumnWidths = void 0;
const calculateOutputColumnWidths = (config) => {
    return config.columns.map((col) => {
        return col.paddingLeft + col.width + col.paddingRight;
    });
};
exports.calculateOutputColumnWidths = calculateOutputColumnWidths;
//# sourceMappingURL=calculateOutputColumnWidths.js.map