import util from './dist/util.js'

export const findPair = util.findPair
export const toJSON = util.toJSON

export const parseMap = util.parseMap
export const parseSeq = util.parseSeq

export const stringifyNumber = util.stringifyNumber
export const stringifyString = util.stringifyString

export const Type = util.Type

export const YAMLError = util.YAMLError
export const YAMLReferenceError = util.YAMLReferenceError
export const YAMLSemanticError = util.YAMLSemanticError
export const YAMLSyntaxError = util.YAMLSyntaxError
export const YAMLWarning = util.YAMLWarning
