import type { SpanningCellConfig } from './types/api';
import type { Row } from './types/internal';
export declare const validateSpanningCellConfig: (rows: Row[], configs: SpanningCellConfig[]) => void;
