import type { SpanningCellConfig, TableUserConfig } from './types/api';
import type { Row } from './types/internal';
export declare const injectHeaderConfig: (rows: Row[], config: TableUserConfig) => [Row[], SpanningCellConfig[]];
