export declare function transform(patterns: string[]): string[];
/**
 * This package only works with forward slashes as a path separator.
 * Because of this, we cannot use the standard `path.normalize` method, because on Windows platform it will use of backslashes.
 */
export declare function removeDuplicateSlashes(pattern: string): string;
