import type { SpanningCellConfig } from './types/api';
import type { Row, Cell } from './types/internal';
export declare const calculateMaximumCellWidth: (cell: Cell) => number;
/**
 * Produces an array of values that describe the largest value length (width) in every column.
 */
export declare const calculateMaximumColumnWidths: (rows: Row[], spanningCellConfigs?: SpanningCellConfig[]) => number[];
