import type { StreamUserConfig } from './types/api';
import type { StreamConfig } from './types/internal';
/**
 * Makes a new configuration object out of the userConfig object
 * using default values for the missing configuration properties.
 */
export declare const makeStreamConfig: (config: StreamUserConfig) => StreamConfig;
