import type { SpanningCellParameters } from './spanningCellManager';
import type { RangeConfig } from './types/internal';
export declare const calculateSpanningCellWidth: (rangeConfig: RangeConfig, dependencies: SpanningCellParameters) => number;
